packer {
  required_plugins {
    windows-update = {
      version = ">= 0.12.0"
      source  = "github.com/rgl/windows-update"
    }
  }
}

source "vsphere-clone" "adds" {
  vcenter_server      = var.vcenter_server
  username            = var.vsphere_username
  password            = var.vsphere_password
  insecure_connection = "true"

  vm_name             = "adds-${var.vm_name}"
  datacenter          = var.vsphere_datacenter
  host                = var.vsphere_host
  folder              = var.vsphere_folder
  datastore           = var.vsphere_datastore

  template            = "Windows-Server-2019-LTSC"

  boot_order          = "disk,cdrom"
  boot_command        = [""]
  boot_wait           = "2m30s"

  communicator        = "winrm"
  winrm_password      = var.winrm_password
  winrm_timeout       = "10m"
  winrm_username      = "administrator"

  RAM                 = 8192
  CPUs                = 2
  
  floppy_files        = [
    "packer/preseed/ADDS/Sysprep_Unattend.xml"
  ]

  shutdown_command    = "C:\\Windows\\System32\\Sysprep\\sysprep.exe /generalize /oobe /unattend:A:\\Sysprep_Unattend.xml"
  shutdown_timeout    = "1h"

  export {
    images            = false
  }
}

build {
  sources = ["source.vsphere-clone.adds"]

  provisioner "powershell" {
    inline = [
      "New-Item -Path 'C:\\Payload\\Scripts' -ItemType 'Directory' -Force:$True -Confirm:$False"
    ]
  }

  provisioner "file" {
    destination = "C:\\Payload\\"
    source      = "scripts/ADDS/payload/"
  }

  provisioner "powershell" {
    scripts = [
      "scripts/ADDS/Install-Prerequisites.ps1",
      "scripts/ADDS/Register-ScheduledTask.ps1"
    ]
  }

  post-processor "shell-local" {
    inline = [
      "pwsh -command \"& scripts/Update-OvfConfiguration.ps1 \\",
      " -OVFFile './output-adds/adds-${var.vm_name}.ovf' \\",
      " -Parameter @{'appliance.name'='ADDS';'appliance.version'='${var.vm_name}'}\"",
      "pwsh -file scripts/Update-Manifest.ps1 \\",
      " -ManifestFileName './output-adds/adds-${var.vm_name}.mf'",
      "ovftool --acceptAllEulas --allowExtraConfig --overwrite \\",
      " './output-adds/adds-${var.vm_name}.ovf' \\",
      " /output/ADDS-appliance.ova"
    ]
  }
  post-processor "shell-local" {
    inline = [
      "pwsh -file scripts/Remove-Resources.ps1 \\",
      " -VMName 'adds-${var.vm_name}' \\",
      " -VSphereFQDN '${var.vcenter_server}' \\",
      " -VSphereUsername '${var.vsphere_username}' \\",
      " -VSpherePassword '${var.vsphere_password}'"
    ]
  }
}